
//   !hpf$ processors p(3, 2)
// 
//   !hpf$ template t(24, 8)
//   !hpf$ distribute t(block, cyclic) onto p
// 
//   integer a(8, 4, 10)
// 
//   !hpf$ align a(i, *, k) with t(4 + 2 * k, i)
// 
//   !hpf$ processors q(2, 3)
// 
//   !hpf$ template s(10, 4)
//   !hpf$ distribute s(block, block) onto q
// 
//   integer b(6, 4, 12)
//   !hpf$ align b(i, j, *) with s(8 - i, j)
// 
//   integer u(6), v(12)
//   !hpf$ align u(i) with b(i, *, *)
// 
//   integer i, j, k
// 
//   do i = 1, 8
//     do j = 1, 4
//       do k = 1, 10
//         a(i, j, k) = 100 * (i - 1) + 10 * (j - 1) + (k - 1)
//       enddo
//     enddo
//   enddo
// 
//   do i = 1, 6
//     u(i) = 1 + mod((i - 1) * 5, 8)
//   enddo
// 
//   do k = 1, 12
//     v(k) = 1 + (k - 1) / 2 + 1
//   enddo
// 
//   b = a(u, :, v) 
// 
//   do i = 1, 6
//     do j = 1, 4
//       do k = 1, 12
//         print "(i3, i3, i3, i10)", i - 1, j - 1, k - 1, b(i, j, k)
//       enddo
//     enddo
//   enddo
// 
// end


#include <stdio.h>
#include "ad++.h"
#include "adrm.h"


int main(int argc, char* argv[]) {
  AdlibInit(argc, argv) ;
  ARMCI_Init() ;

  Procs2 p(3, 2) ;

  CyclicRange x(8, p.dim(1)) ;
  CollapsedRange y(4) ;
  Range z = BlockRange(24, p.dim(0)).subrng(10, 5, 2) ;

  ArrayRM3<int> a(x, y, z, p) ;

  Procs2 q(2, 3) ;

  Range r = BlockRange(10, q.dim(0)).subrng(6, 6, -1) ;
  BlockRange s(4, q.dim(1)) ;
  CollapsedRange t(12) ;

  Array3<int> b(r, s, t, q) ;
  Array1<int> u(r, q) ;
  Array1<int> v(t, q) ;

  ON(p) {
    Index ix(x), iy(y), iz(z) ;
    OVERALL(ix) {
      OVERALL(iy) {
	OVERALL(iz) {
	  a(ix, iy, iz) = 100 * ix + 10 * iy + iz ;
	} ALLOVER(iz) ;
      } ALLOVER(iy) ;
    } ALLOVER(ix) ;
  } NO(p) ;

  ON(q) {
    Index ir(r) ;
    OVERALL(ir) {
      u(ir) = (ir * 5) % 8 ;
    } ALLOVER(ir) ;

    Index it(t) ;
    OVERALL(it) {
      v(it) = it / 2 + 1 ;
    } ALLOVER(it) ;
  } NO(q) ;

  vecGatherRM(b, a, &u, (Section1<int>*) 0, &v) ;

  gprintf("After vecGather...\n") ;
  aprintf("%3R0%3R1%3R2%10d\n", b) ;

  ARMCI_Finalize() ;
  AdlibFinalize() ;
}


